#include "topwin.h"
#include <iostream>

TopWin::~TopWin() {}

TopWin::TopWin() : 
  //initialize controlhbox widgets
  controlhbox(false, 0),
  seekvbox(false, 0),
  seeklabel("0:00 - 0:00"),
  progress(0.00, 0.00, 0.00, 0.00, 0.00, 0.00),
  volumeadj(0.70, 0.00, 1.00, 0.01, 0.00, 0.00),
  hscale(progress),
  
  //infolabel
  infolabel("0 item(s) - 0.0 minutes - 0.00 MB")
{
  //set title for mainwindow
  set_title("Dingo Music Manager");
 
  //create menubar's widgets
  menubar_action = Gtk::ActionGroup::create();
  
  //File menu:
  menubar_action->add(Gtk::Action::create("FileMenu", "File"));
  //Sub Edit menu:
  menubar_action->add(Gtk::Action::create("FileImportMedia", "Import Media"));
  menubar_action->add(Gtk::Action::create("FileImportPlaylist", "Import Playlist"));
  menubar_action->add(Gtk::Action::create("FileClose", Gtk::Stock::CLOSE));
  menubar_action->add(Gtk::Action::create("FileQuit", Gtk::Stock::QUIT));
  
  //Edit menu:
  menubar_action->add(Gtk::Action::create("EditMenu", "Edit"));
  //Sub Edit menu:
  menubar_action->add(Gtk::Action::create("EditSelectAll", "Select All"));
  menubar_action->add(Gtk::Action::create("EditSelectNone", "Select None"));
  menubar_action->add(Gtk::Action::create("EditRemoveLib", Gtk::Stock::REMOVE, "Remove From Library", "EditRemoveLib"));
  menubar_action->add(Gtk::Action::create("EditRemoveDrive", "Delete From Drive"));
  menubar_action->add(Gtk::Action::create("EditOpenFolder", "Open Containing Folder"));
  menubar_action->add(Gtk::Action::create("EditProperties", Gtk::Stock::PROPERTIES));
  menubar_action->add(Gtk::Action::create("EditPreferences", Gtk::Stock::PREFERENCES));
  
  //View menu:
  menubar_action->add(Gtk::Action::create("ViewMenu", "View"));
  //Sub View menu:
  menubar_action->add(Gtk::Action::create("ViewSub", "Subtitles"));
  menubar_action->add(Gtk::ToggleAction::create("ViewBrowser", "Show Browser", "ViewBrowser", true));
  menubar_action->add(Gtk::ToggleAction::create("ViewVideoArea", "Show Video Area", "ViewVideoArea", true));
  menubar_action->add(Gtk::ToggleAction::create("ViewFullScreen", "Full Screen", "ViewFullScreen", false));
  
  //Play menu:
  menubar_action->add(Gtk::Action::create("PlayMenu", "Play"));
  //Sub Play menu:
  menubar_action->add(Gtk::Action::create("PlayPlay", Gtk::Stock::MEDIA_PLAY));
  menubar_action->add(Gtk::ToggleAction::create("PlayStopAfterFinish", "Stop After Finish", "PlayStopAfterFinish", false));
  menubar_action->add(Gtk::Action::create("PlayNext", Gtk::Stock::MEDIA_NEXT));
  menubar_action->add(Gtk::Action::create("PlayPrev", Gtk::Stock::MEDIA_PREVIOUS));
  menubar_action->add(Gtk::Action::create("PlayRepeatMode", "Repeat Mode"));
  //Sub Repeat Mode menu:
  Gtk::RadioAction::Group repeatgroup;
  menubar_action->add(Gtk::RadioAction::create(repeatgroup, "RepeatOff", "Turn Off Repeat"));
  menubar_action->add(Gtk::RadioAction::create(repeatgroup, "RepeatSingle", "Repeat Single"));
  menubar_action->add(Gtk::RadioAction::create(repeatgroup, "RepeatAll", "Repeat All"));
  
  //Help menu:
  menubar_action->add(Gtk::Action::create("HelpMenu", "Help"));
  menubar_action->add(Gtk::Action::create("HelpAbout", Gtk::Stock::ABOUT));
  
  //Layout the actions in menubar:
  Glib::ustring ui_info = 
    "<ui>"
    "  <menubar name='MenuBar'>"
    "    <menu action='FileMenu'>"
    "     <menuitem action='FileImportMedia'/>"
    "      <menuitem action='FileImportPlaylist'/>"
    "      <separator/>"
    "      <menuitem action='FileClose'/>"
    "      <menuitem action='FileQuit'/>"
    "    </menu>"
    "    <menu action='EditMenu'>"
    "      <menuitem action='EditSelectAll'/>"
    "      <menuitem action='EditSelectNone'/>"
    "      <separator/>"
    "      <menuitem action='EditRemoveLib'/>"
    "      <menuitem action='EditRemoveDrive'/>"
    "      <separator/>"
    "      <menuitem action='EditOpenFolder'/>"
    "      <separator/>"
    "      <menuitem action='EditProperties'/>"
    "      <separator/>"
    "      <menuitem action='EditPreferences'/>"
    "    </menu>"
    "    <menu action='ViewMenu'>"
    "      <menuitem action='ViewSub'/>"
    "      <separator/>"
    "      <menuitem action='ViewBrowser'/>"
    "      <menuitem action='ViewVideoArea'/>"
    "      <separator/>"
    "      <menuitem action='ViewFullScreen'/>"
    "    </menu>"
    "    <menu action='PlayMenu'>"
    "      <menuitem action='PlayPlay'/>"
    "      <menuitem action='PlayStopAfterFinish'/>"
    "      <separator/>"
    "      <menuitem action='PlayNext'/>"
    "      <menuitem action='PlayPrev'/>"
    "      <separator/>"
    "      <menu action = 'PlayRepeatMode'>"
    "        <menuitem action='RepeatOff'/>"
    "        <menuitem action='RepeatSingle'/>"
    "        <menuitem action='RepeatAll'/>"
    "      </menu>"
    "    </menu>"
    "    <menu action='HelpMenu'>"
    "      <menuitem action='HelpAbout'/>"
    "    </menu>"
    "  </menubar>"
    "</ui>";

#ifdef GLIBMM_EXCEPTIONS_ENABLED
try {
  menubar_ui->add_ui_from_string(ui_info);
}
catch (const Glib::Error& ex){
  std::cerr << "building menus failed" << ex.what();
}

#else
std::auto_ptr<Glib::Error> ex;
menubar_ui->add_ui_from_string(ui_info, ex);
if (ex.get()) {
  std::cerr << "building menu failed: " << ex->what();
}
#endif //GLIBMM_EXCEPTIONS_ENABLED

  Gtk::Widget* pmenubar = menubar_ui->get_widget("/MenuBar");
  
  //create all controlhbox buttons
  playbutton.property_image() = new Gtk::Image(Gtk::Stock::MEDIA_PLAY, Gtk::ICON_SIZE_SMALL_TOOLBAR);
  playbutton.set_relief(Gtk::RELIEF_NONE);
  
  prevbutton.property_image() = new Gtk::Image(Gtk::Stock::MEDIA_PREVIOUS, Gtk::ICON_SIZE_SMALL_TOOLBAR);
  prevbutton.set_relief(Gtk::RELIEF_NONE);
  
  nextbutton.property_image() = new Gtk::Image(Gtk::Stock::MEDIA_NEXT, Gtk::ICON_SIZE_SMALL_TOOLBAR);
  nextbutton.set_relief(Gtk::RELIEF_NONE);
  
  repeatbutton.property_image() = new Gtk::Image(Gtk::Stock::REFRESH, Gtk::ICON_SIZE_SMALL_TOOLBAR);
  repeatbutton.set_relief(Gtk::RELIEF_NONE);
  
  fullbutton.property_image() = new Gtk::Image(Gtk::Stock::FULLSCREEN, Gtk::ICON_SIZE_SMALL_TOOLBAR);
  fullbutton.set_relief(Gtk::RELIEF_NONE);
  
  volumebutton.set_adjustment(volumeadj);
  
  hscale.set_draw_value(false);
  hscale.set_size_request(200, 20);
  
  seeklabel.set_size_request(70, 15);
  
  //pack widgets into seekvbox
  seekvbox.pack_start(hscale, Gtk::PACK_SHRINK);
  seekvbox.pack_start(seeklabel, Gtk::PACK_SHRINK);
  
  //pack widgets into controlhbox
  controlhbox.pack_start(prevbutton, Gtk::PACK_SHRINK);
  controlhbox.pack_start(playbutton, Gtk::PACK_SHRINK);
  controlhbox.pack_start(nextbutton, Gtk::PACK_SHRINK);
  controlhbox.pack_start(seekvbox, Gtk::PACK_SHRINK);
  
  controlhbox.pack_end(volumebutton, Gtk::PACK_SHRINK);
  controlhbox.pack_end(searchbox, Gtk::PACK_SHRINK);
  controlhbox.pack_end(fullbutton, Gtk::PACK_SHRINK);
  controlhbox.pack_end(repeatbutton, Gtk::PACK_SHRINK);
  
  //request size for videoarea
  videoarea.set_size_request(600, 335);
  
  //create l_scroll
  l_model = Gtk::TreeStore::create(l_colrec);
  l_treeview.set_model(l_model);
  
  Gtk::TreeModel::Row row = *(l_model->append());
  row[l_colrec.l_icon] = Gdk::Pixbuf::create_from_file("now-playing.png");
  row[l_colrec.l_status] = "Now Playing";
  
  row = *(l_model->append());
  row[l_colrec.l_icon] = Gdk::Pixbuf::create_from_file("file-queue.png");
  row[l_colrec.l_status] = "Play Queue";
  
  row = *(l_model->append());
  row[l_colrec.l_icon] = Gdk::Pixbuf::create_from_file("local-media.png");
  row[l_colrec.l_status] = "Local Media";
  
  Gtk::TreeModel::Row childrow = *(l_model->append(row.children()));
  childrow[l_colrec.l_icon] = Gdk::Pixbuf::create_from_file("music.png");
  childrow[l_colrec.l_status] = "Music";
  
  childrow = *(l_model->append(row.children()));
  childrow[l_colrec.l_icon] = Gdk::Pixbuf::create_from_file("video.png");
  childrow[l_colrec.l_status] = "Videos";
  
  row = *(l_model->append());
  row[l_colrec.l_icon] = Gdk::Pixbuf::create_from_file("playlist.png");
  row[l_colrec.l_status] = "Playlists";
  
  l_treeview.append_column(" ", l_colrec.l_expander);
  l_treeview.append_column("Icon", l_colrec.l_icon);
  l_treeview.append_column("Actions", l_colrec.l_status);
  
  l_treeview.get_column(0)->set_expand(false);
  l_treeview.get_column(0)->set_max_width(20);
  
  l_treeview.get_column(1)->set_expand(false);
  l_treeview.get_column(1)->set_min_width(50);
  
  l_treeview.get_column(2)->set_expand(false);
  l_treeview.get_column(2)->set_min_width(50);
  
  l_scroll.add(l_treeview);
  l_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  l_scroll.set_size_request(170, 600);
  
  //create mtl_scroll
  mtl_model = Gtk::ListStore::create(mtl_colrec);
  mtl_treeview.set_model(mtl_model);
  
  row = *(mtl_model->append());
  row[mtl_colrec.mtl_artist] = "All Artists (0)";
  
  mtl_treeview.append_column("Artists", mtl_colrec.mtl_artist);
  
  mtl_scroll.add(mtl_treeview);
  mtl_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  mtl_scroll.set_size_request(310, 335);
  
  //create mtr_scroll
  mtr_model = Gtk::ListStore::create(mtr_colrec);
  mtr_treeview.set_model(mtr_model);
  
  row = *(mtr_model->append());
  row[mtr_colrec.mtr_album] = "All Albums (0)";
  
  mtr_treeview.append_column("Albums", mtr_colrec.mtr_album);
  
  mtr_scroll.add(mtr_treeview);
  mtr_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  mtr_scroll.set_size_request(310, 335);
  
  //create rt_scroll
  rt_model = Gtk::ListStore::create(rt_colrec);
  rt_treeview.set_model(rt_model);
  
  row = *(rt_model->append());
  row[rt_colrec.rt_artist] = "All Artists (0)";
  
  rt_treeview.append_column("Artists", rt_colrec.rt_artist);
  
  rt_scroll.add(rt_treeview);
  rt_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  rt_scroll.set_size_request(220, 290);
  
  //create rb_scroll
  rb_model = Gtk::ListStore::create(rb_colrec);
  rb_treeview.set_model(rb_model);
  
  row = *(rb_model->append());
  row[rb_colrec.rb_album] = "All Albums (0)";
  
  rb_treeview.append_column("Albums", rb_colrec.rb_album);
  
  rb_scroll.add(rb_treeview);
  rb_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  rb_scroll.set_size_request(220, 300);
  
  //create mb_scroll
  mb_model = Gtk::ListStore::create(mb_colrec);
  mb_treeview.set_model(mb_model);
  
  mb_treeview.append_column(" ", mb_colrec.mb_status);
  mb_treeview.append_column("Name", mb_colrec.mb_name);
  mb_treeview.append_column("Artist", mb_colrec.mb_artist);
  mb_treeview.append_column("Album", mb_colrec.mb_album);
  mb_treeview.append_column("Time", mb_colrec.mb_time);
  
  mb_treeview.get_column(0)->set_min_width(24);
  mb_treeview.get_column(0)->set_expand(false);
  mb_treeview.get_column(0)->set_resizable(false);
  mb_treeview.get_column(1)->set_min_width(80);
  mb_treeview.get_column(1)->set_expand(true);
  mb_treeview.get_column(1)->set_resizable(true);
  mb_treeview.get_column(2)->set_min_width(50);
  mb_treeview.get_column(2)->set_expand(true);
  mb_treeview.get_column(2)->set_resizable(true);
  mb_treeview.get_column(3)->set_min_width(50);
  mb_treeview.get_column(3)->set_expand(true);
  mb_treeview.get_column(3)->set_resizable(true);
  mb_treeview.get_column(4)->set_min_width(70);
  mb_treeview.get_column(4)->set_expand(false);
  mb_treeview.get_column(4)->set_resizable(false);
  
  mb_scroll.add(mb_treeview);
  mb_scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  mb_scroll.set_size_request(600, 200);
  
  //pack all widgets in containers
  rvpaned.pack1(rt_scroll, Gtk::SHRINK);
  rvpaned.pack2(rb_scroll, Gtk::SHRINK);
  
  submhpaned.pack1(mtl_scroll, Gtk::EXPAND);
  submhpaned.pack2(mtr_scroll, Gtk::EXPAND);
  
  mhpaned.pack1(videoarea, Gtk::EXPAND);
  mhpaned.pack2(submhpaned, Gtk::SHRINK);
  
  mvpaned.pack1(mhpaned, Gtk::FILL);
  mvpaned.pack2(mb_scroll, Gtk::FILL);
  
  subhpaned.pack1(mvpaned, Gtk::FILL);
  subhpaned.pack2(rvpaned, Gtk::SHRINK);
  
  tophpaned.pack1(l_scroll, Gtk::SHRINK);
  tophpaned.pack2(subhpaned, Gtk::EXPAND);
  
  topvbox.pack_start(*pmenubar, Gtk::PACK_SHRINK);
  topvbox.pack_start(controlhbox, Gtk::PACK_SHRINK);
  topvbox.pack_start(tophpaned, Gtk::PACK_EXPAND_WIDGET);
  topvbox.pack_start(infolabel, Gtk::PACK_SHRINK);
  
  add(topvbox);
  
  show_all_children();
  submhpaned.hide();
}
  
